package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcCall;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Repository
public class EmployeeDaoImpl implements EmployeeDao {
    @Autowired
    private DataSource dataSource;
    private JdbcTemplate jdbcTemplate;
    private SimpleJdbcCall simpleJdbcCall;

    @PostConstruct
    private void postConstruct() {
        jdbcTemplate = new JdbcTemplate(dataSource);
        simpleJdbcCall = new SimpleJdbcCall(jdbcTemplate)
                .withProcedureName("get_managers")
                .returningResultSet("manager_cursor",
                        BeanPropertyRowMapper.newInstance(Employee.class));
    }

    @Override
    public void save(Employee employee) {
        String sql = "insert into EMPLOYEE (ID, NAME, ROLE) values (seq_employee.nextval, ?, ?)";
        jdbcTemplate.update(sql, employee.getName(),
                employee.getRole());
    }

    @Override
    public Employee load(long id) {
        List<Employee> list = jdbcTemplate.query("select * from EMPLOYEE where id =?",
                new Object[]{id}, (resultSet, i) -> {
                    return toEmployee(resultSet);
                });

        if (list.size() == 1) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public void delete(long id) {
        jdbcTemplate.update("delete from Employee where id = ?", id);
    }

    @Override
    public void update(Employee employee) {
        //todo
    }

    @Override
    public List<Employee> loadAll() {
        return jdbcTemplate.query("select * from Employee", (resultSet, i) -> {
            return toEmployee(resultSet);
        });
    }

    private Employee toEmployee(ResultSet resultSet) throws SQLException {
        Employee em = new Employee();
        em.setId(resultSet.getLong("ID"));
        em.setName(resultSet.getString("NAME"));
        em.setRole(resultSet.getString("ROLE"));
        return em;
    }

    @Override
    public List<Employee> getAllManagers() {
        Map<String, Object> result =
                simpleJdbcCall.execute(new HashMap<String, Object>(0));
        System.out.println("result: " + result);
        return (List) result.get("manager_cursor");
    }
}